from typing import Sequence, Optional

from RocketSim import BallState, CarState, GameMode, CarConfig

type TVec3 = tuple[float, float, float]
"""
The items are (X, Y, Z) respectively
"""
type TRotmat = tuple[TVec3, TVec3, TVec3]
"""
The items are (forward, right, up) respectively
"""
type TBall = tuple[TVec3, TRotmat, TVec3, TVec3]
"""
The items are (location, rotation, velocity, angular velocity) respectively
"""
type TCar = tuple[int, TVec3, TRotmat, TVec3, TVec3, float, bool, bool, bool, float]
"""
The items are (car_id, location, rotation, velocity, angular velocity, boost, has jumped, has double jumped, has flipped, demo respawn timer) respectively
"""

def set_boost_pad_locations(locations: Sequence[TVec3]) -> ...:
    pass

def get_state_set() -> Optional[tuple[Sequence[float], TBall, Sequence[TCar]]]:
    """
    Sequence[float] - Boost pad states, 0 for full and some positive value for the time in seconds until it respawns
    TBall - Ball state
    Sequence[TCar] - Car states
    """
    pass

def get_game_speed() -> float:
    """
    Returns the current wanted game speed. Default is 1.0 if there has been no request from RLViser to change it.

    The game speed is a multiplier of the game speed, 1.0 is normal speed, 2.0 is double speed, etc.
    """
    pass

def get_game_paused() -> bool:
    """
    Returns the current wanted game pause state. Default is False if there has been no request from RLViser to change it.

    True means the game is paused, False means the game is unpaused.
    """

def report_game_speed(speed: float) -> ...:
    """
    Reports the current game speed to RLViser. This is used to update the game speed in the UI and to properly interpolate the game state.

    NOTE: This is only needed when RLViser did not request the game speed change.
    """
    pass

def report_game_paused(paused: bool) -> ...:
    """
    Reports the current game pause state to RLViser. This is used to update the game pause state in the UI.

    NOTE: This is only needed when RLViser did not request the game pause state change.
    """
    pass

type CarData = tuple[int, int, CarConfig, CarState]
"""
The items are (car_id, team, car_config, car_state) respectively
"""

def render(tick_count: int, tick_rate: float, game_mode: GameMode, boost_pad_states: Sequence[bool], ball: BallState, cars: Sequence[CarData]) -> ...:
    pass

def launch():
    """
    Opens RLViser. Useful if `quit()` was called and you want to open it again.
    """

def quit() -> ...:
    pass



import time
from itertools import chain

import numpy as np
import rlviser_py
from rlgym.api import RLGym
from rlgym.rocket_league.action_parsers import LookupTableAction, RepeatAction
from rlgym.rocket_league.done_conditions import (
    AnyCondition,
    GoalCondition,
    NoTouchTimeoutCondition,
    TimeoutCondition,
)
from rlgym.rocket_league.obs_builders import DefaultObs
from rlgym.rocket_league.reward_functions import CombinedReward, GoalReward, TouchReward
from rlgym.rocket_league.sim import RocketSimEngine
from rlgym.rocket_league.state_mutators import (
    FixedTeamSizeMutator,
    KickoffMutator,
    MutatorSequence,
)

from gym_renderer import RLViserRenderer

if __name__ == "__main__":
    game_speed = 1

    env = RLGym(
        state_mutator=MutatorSequence(
            FixedTeamSizeMutator(blue_size=2, orange_size=2), KickoffMutator()
        ),
        obs_builder=DefaultObs(zero_padding=2),
        action_parser=RepeatAction(LookupTableAction()),
        reward_fn=CombinedReward((GoalReward(), 10.0), (TouchReward(), 0.1)),
        termination_cond=GoalCondition(),
        truncation_cond=AnyCondition(
            TimeoutCondition(300.0), NoTouchTimeoutCondition(30.0)
        ),
        transition_engine=RocketSimEngine(),
        renderer=RLViserRenderer(),
    )

    # simulate 2 episodes
    for _ in range(2):
        obs_dict = env.reset()
        steps = 0
        ep_reward = {agent_id: 0.0 for agent_id in env.agents}
        t0 = time.time()
        while True:
            actions = {}
            for agent_id, (type, action_spaces) in env.action_spaces.items():
                actions[agent_id] = np.random.randint(action_spaces, size=(1,))

            obs_dict, reward_dict, terminated_dict, truncated_dict = env.step(
                actions
            )
            env.render()
            time.sleep(15 / 120 / game_speed)
            steps += 1

            for agent_id, reward in reward_dict.items():
                ep_reward[agent_id] += reward

            if any(chain(terminated_dict.values(), truncated_dict.values())):
                break

            game_speed = rlviser_py.get_game_speed()

        ep_time = time.time() - t0
        print(
            f"Steps per second: {steps / ep_time:.0f} | Episode time: {ep_time:.2f} | Episode Reward: {max(ep_reward.values()):.2f}"
        )

    env.close()
